#! /bin/bash
# Copyright (C) 1999, 2000, 2001 by SuSE GmbH.
# Author: Karl Eichwalder <ke@suse.de>, 1999-2001.
# GPL
# Based on ideas by Mark Galassi <rosalia@cygnus.com> (1999).

package=docbook-toys
version=1.51.0

LANGUAGE=C; export LANGUAGE
LC_ALL=C; export LC_ALL

SGML_CATALOG_FILES=
# SP_ENCODING=
[ -z "$SP_CHARSET_FIXED" ] || SP_CHARSET_FIXED=YES
DIR=.

jade_prefix=o

progname=${0##*/}
usage="\
Usage: $progname [OPTION]... FILE.sgml
Run a DocBook SGML document FILE.sgml through various programs.

If a long option shows an argument as mandatory, then it is mandatory
for the equivalent short option also.  Similarly for optional arguments.

Options:
  --help                  display this help and exit
  -v, --version           output version information and exit

  -x                      trace the shell script

Output formats:
  -d, --dvi               output DVI
  -f, --fot               output FOT
  -h, --html              output HTML
  -m, --mif               output MIF
  --pdf                   output PDF
  -p, --ps, --postscript  output PostScript
  -r, --rtf               output RTF
  -t, --tex               output TeX
  -X, --xml               output XML (DocBook)
      --xencoding VAR     set XML output encoding (default: iso-8859-1)
      --xoptions VAR      add XML output options; e.g., \"-xempty -xlower\"
                            (default)

HTML:
  --nochunks              don't produce HTML subdocuments; output to stdout

General options:
  -S, --strict            use the SGML declaration coming with the DocBook DTDs
                            (no minimizations are allowed)
  -j, --jade VAR...       set \"jade\" variables; e.g., \"-V %shade-verbatim%\"
  -o, --output DIR        set output DIR
  -J, --use-jade          use jade instead of openjade
  -c, --catalog FILE      use FILE as the catalog
  -s, --style FILE        use FILE as driver style
  -V, --validate          validate (sloppy); without \"--strict\" allow
                            minimizations
  -W, --validate-strict   validate (strict); implies \"--strict\"

Only the last specified output format will be produced.

The script will automatically use FILE.dsl as a driver style if it is found in
the same directory as FILE.sgml.

DocBook XML version 4 input files will be properly processed, too.

Please, report bugs to Karl Eichwalder <keichwa@gmx.net>."

if [ $# -eq 0 ]; then echo "$usage"; exit 1; fi

while test $# -gt 0; do
  case $1 in
    -d | --dvi)  mode=db2dvi; shift ;;
    -f | --fot)  mode=db2fot; shift ;;
    -h | --html) mode=db2html; shift ;;
    -m | --mif)  mode=db2mif;  shift ;;
    -p | --ps | --postscript)   mode=db2ps;  shift ;;
    --pdf)       mode=db2pdf; shift ;;
    -r | --rtf)  mode=db2rtf; shift ;;
    -t | --tex)  mode=db2tex; shift ;;
    -X | --xml)  mode=db2xml; shift ;;
    # --xencoding)
    --xen*) if test $# -gt 2; then XMLENCODING=$2; shift 2;
                 else echo "$usage"; exit 1; fi ;;
    # --xoptions)
    --xop*) if test $# -gt 2; then XMLOPTIONS=$2; shift 2;
                 else echo "$usage"; exit 1; fi ;;
    -j | --jade) if test $# -gt 2; then JADEVARS=$2; shift 2;
                 else echo "$usage"; exit 1; fi ;;
    # for compatibility
    #-o | --use-open*) true; shift ;;
    -o | --out*) if test $# -gt 2; then [ -n "$2" ] && DIR="$2"; shift 2;
      else echo "$usage"; exit 1; fi ;;
    # --nochunks ) nochunks=yes; verbose="" ; JADEHTML='-V nochunks'; shift;;
    --noc* ) nochunks=yes; quiet=yes ; JADEHTML='-V nochunks'; shift;;
    # --catalog) if test $# -gt 2; then  MYSTYLE=$2; shift 2;
    -c | --ca*) if test $# -gt 2; then  MYCAT=$2; shift 2;
                 else echo "$usage"; exit 1; fi ;;
    -s | --style) if test $# -gt 2; then  MYSTYLE=$2; shift 2;
                 else echo "$usage"; exit 1; fi ;;
    -x) trace=-x; set -x; shift ;;
    -q | --quiet ) quiet=yes ; shift ;;
    -S | --str* ) strict=yes ; shift ;;
    -V | --validate)  mode=validate; shift ;;
    -W | --validate-s*)  mode=validate; strict=yes; shift ;;
    -v | --ve* ) echo "$progname - $package $version"; exit 0 ;;
    --h* ) echo "$usage"; exit 0 ;;
    # -) break ;;
    -*) echo "Try '$progname --help' for more information."; exit 1 ;;
    *) break
  esac
done

# echo "$progname"
if [ ! "$progname" = "db2x.sh" ]; then
  [ -z "$mode" ] && mode="$progname"
elif [ -z "$mode" ]; then
  echo "$usage"; exit 1
fi

prefix=/usr
datadir=/usr/share
# CATALOG=/usr/share/sgml/CATALOG.docbkdsl
CATALOG=/usr/share/sgml/CATALOG.docbook-dsssl-stylesheets
CATALOG_JADE=/usr/share/sgml/CATALOG.openjade
htmldsl=/usr/share/sgml/docbook/dsssl-stylesheets/html/docbook.dsl
dslhome=${htmldsl%/html/docbook.dsl}
sgmldir=$datadir/sgml

JADE=openjade
# ${jade_prefix:+o}
NSGMLS=${jade_prefix}nsgmls
SGMLNORM=${jade_prefix}sgmlnorm
SGML2XML=${jade_prefix}sx

WHICH='type -p'

case "$1" in
/*) file="$1" ;;
*)  file="$PWD/$1" ;;
esac

if [ $mode != db2html ]; then
  # echo $mode
  if ! cd "$DIR"; then
    echo "Cannot change to directory \"$DIR\".  Exiting." >&2
    exit 1
  fi
fi

# echo $file
if [ ! -r "$file" ]; then
  echo Cannot read \"$file\".  Exiting. >&2
  exit 1
fi

if [ x"$quiet" != xyes ]; then
  quiet=""
fi

dirbasefile="${file%\.*}"
basefile="${dirbasefile##*/}"
suffix=${file##*\.}

SGML_DECL_SLOPPY=$sgmldir/docbook/$package/docbook-sloppy.dcl
if [ -n "$strict" ]; then
  SGML_DECL_SLOPPY=
elif [ ! -r $SGML_DECL_SLOPPY ] ; then
  SGML_DECL_SLOPPY=
fi

detect_xml_encoding () {
  local encoding
  local version
  eval $(sed 100q $file | sed -n 'p;/\?>/q' \
    | awk '
/<\?xml/ { f=1;printf "%s ", $0;next}
/\?>/{print;exit}
f==1{printf "%s ", $0} ' | sed 's/<?xml//;s/?>//')
  [ -z "$SP_ENCODING" ] && export SP_ENCODING=$encoding
  [ -z "$SP_ENCODING" ] && export SP_ENCODING=UTF-8
  if [ "$SP_ENCODING" = UTF-8 -o "$SP_ENCODING" = utf-8 ]; then
    export SP_CHARSET_FIXED=YES
  fi
  # echo $SP_ENCODING $SP_CHARSET_FIXED
}

case $suffix in
  sgm | sgml) flavor=sgml ;;
  xml) flavor=xml ;;
  *) if file "$file" | cut -d: -f 2 | grep -q XML; then
    flavor=xml
    else
    flavor=sgml
    fi ;;
    # echo $"Filename suffix \"$suffix\" ambiguous: $file" ; exit 1 ;;
esac

# SuSE Linux >= 7.2
if [ -r $CATALOG_JADE ]; then
  CATALOG="/usr/share/sgml/CATALOG.iso_ent -c $CATALOG_JADE -c $CATALOG"
fi

if [ -n "$MYCAT" ]; then
  [ -r "$MYCAT" ] || { echo "$0: $MYCAT" not found ; exit 1 ; }
  if [ -f "/$MYCAT" ]; then
    CATALOG="$MYCAT"
  else
    CATALOG=$(pwd)"/$MYCAT"
  fi
fi

# sgmldir=/usr/share/sgml
CAT_SGML=$(find $sgmldir -maxdepth 1 -name 'CATALOG.docbook_[3-9]' \
                         -printf '%p ' 2>/dev/null)
if [ -z "$CAT_SGML" ]; then
  # search for the old catalog files (SL <= 7.1)
  CAT_SGML=$(find $sgmldir -maxdepth 1 -name 'CATALOG.docbk[34]?' \
                           -printf '%p ' 2>/dev/null)
fi
CAT_GNOME=$(find $sgmldir -maxdepth 1 -name 'CATALOG.gnome' \
                         -printf '%p ' 2>/dev/null)
# echo $CAT_SGML
# Fallback or better use /dev/null?
[ -z "$CAT_SGML" ] && CAT_SGML="$CATALOG"
for j in $CAT_SGML $CAT_GNOME; do
  [ -r "$j" ] || continue
  CAT_SGML_JADE="-c $j $CAT_SGML_JADE"
  CAT_SGML_NSGMLS="-m $j $CAT_SGML_NSGMLS"
done

# avoid 3x catalog
CAT_XML=$(find $sgmldir -maxdepth 1 -name 'CATALOG.db4*xml' | sort -r)
#                        -printf '%p ' 2>/dev/null)
# Fallback or better use /dev/null?
[ -z "$CAT_XML" ] && CAT_XML="$CATALOG"
for j in $CAT_XML; do
  [ -r "$j" ] || continue
  CAT_XML_JADE="-c $j $CAT_XML_JADE"
  CAT_NSGMLS="-m $j $CAT_NSGMLS"
done

# STYLE_HTML=html/docbook.dsl
# STYLE=${STYLE_HTML}
if [ -n "$MYSTYLE" ]; then
  [ -r "$MYSTYLE" ] || { echo "$0: $MYSTYLE" not found ; exit 1 ; }
  if [ -f "/$MYSTYLE" ]; then
    STYLE="$MYSTYLE"
  else
    STYLE=$(pwd)"/$MYSTYLE"
  fi
elif [ -r $basefile.dsl ] ; then
  STYLE=$(pwd)"/$basefile.dsl"
  [ -z "$quiet" ] && echo using style file $STYLE
else
  # [ -r suse-both.dsl ] || cp ${sgmldir}/docbook/${package}/suse-both.dsl .
  # STYLE=$(pwd)/suse-both.dsl
  STYLE=${sgmldir}/docbook/${package}/suse-both.dsl
fi

HTML_STYLESHEET=${dslhome}/frames/docbook.css
ADMON_GRAPHICS=${dslhome}/images/*.gif

case "$flavor" in
  sgml) _CAT_JADE=$CAT_SGML_JADE ;;
  xml) detect_xml_encoding
    # convert to UTF-8; does not work automatically?
    # SP_CHARSET_FIXED=YES
    # SP_ENCODING=XML
    # echo $SP_ENCODING
    if [ -r /usr/share/sgml/openjade/xml.dcl ]; then
      SGML_DECL_SLOPPY=/usr/share/sgml/openjade/xml.dcl
    elif [ -r /usr/share/sgml/jade_dsl/xml.dcl ]; then
      SGML_DECL_SLOPPY=/usr/share/sgml/jade_dsl/xml.dcl
    elif [ -r /usr/share/sgml/jade_dsl/xml.decl ]; then
    SGML_DECL_SLOPPY=/usr/share/sgml/jade_dsl/xml.decl
    else
      echo "XML declaration is missing." ; exit 1
    fi
    _CAT_JADE=$CAT_XML_JADE ;;
  *) echo "flavor not detected.  Must not happen...";  exit 1;;
esac

create_or_check_style_files () {
  if [ $STYLE != ${sgmldir}/docbook/${package}/suse-both.dsl ]; then
    # using private style file; thus:
    true
  else
    if [ ! -f suse-html.dsl ]; then
      sed "s/@encoding@/$SP_ENCODING/" ${sgmldir}/docbook/${package}/suse-html.dsl \
	> suse-html.dsl
      [ -r CATALOG.local ] || {
	echo "SYSTEM suse-html.dsl \"$(pwd)/suse-html.dsl\""
	echo "SYSTEM suse-print.dsl \"$(pwd)/suse-print.dsl\""
      } > CATALOG.local
    else
      if [ "$flavor" = xml ]; then
	if ! grep -q -i -e "charset=.*$SP_ENCODING" suse-html.dsl; then
	  echo "\
file encoding and setting in stylesheet do not match:
$file: $SP_ENCODING
suse-html.dsl: $(grep -n -H charset= suse-html.dsl)"
	  sleep 3
	fi
      fi
    fi

    if [ ! -f suse-print.dsl ]; then
      cp ${sgmldir}/docbook/${package}/suse-print.dsl .
    fi
  fi
}

create_or_check_style_files

[ -r $(pwd)/CATALOG.local ] && CATALOG="$(pwd)/CATALOG.local -c $CATALOG"

extract_dtd_declaration(){
  # The DTD parser is limited:
  #    first 10 lines
  #    comments ("-- ... --") can make it fail easily
  dtd_declaration=$(head -n 10 $file \
    | while read l ; do printf "%s " $l ; done \
    | sed -e 's:.*\(<!doctype\):\1:I' \
          -e 's:\[.*:>:' \
          -e 's:[[:space:]]+: :' \
          -e 's: *>.*:>:')
  echo $"DTD declaration is: $dtd_declaration"
  DOCTYPE=${dtd_declaration#* }
  DOCTYPE=${DOCTYPE%% *}
  doctype=$(echo $DOCTYPE | tr '[:upper:]' '[:lower:]')
}

gen_outfile(){
  output=$basefile.$end
  output=${output%.}
  outputlog=$basefile.log
}

jade_error () {
  echo "running $JADE failed; check style files included by:"
  echo "${STYLE}"
  echo "aborting."
  exit 1
}

do_dvi(){
  texfile=$output
  end=dvi
  gen_outfile
  echo tex file name is "$texfile"
  echo dvi file name is "$output"
  jadetex "$texfile"

  # if there are unresolved references, re-run jadetex, twice
  if egrep '^LaTeX Warning: There were undefined references.$' ${outputlog} >/dev/null 2>&1
  then
      jadetex "$texfile"
      jadetex "$texfile"
  fi
} # do_dvi

do_fot(){
  end=fot
  gen_outfile
  echo fot output file name is "$output"
  $JADE ${JADEVARS} -t fot -c $CATALOG $_CAT_JADE \
    -D $dslhome -d ${STYLE}\#print \
    -o "$output" $SGML_DECL_SLOPPY "$file" || jade_error
} # do_fot


do_html(){
  #if echo "$file" | egrep -i '\.sgml$|\.sgm$|\.xml$' >/dev/null 2>&1; then
    end=
    [ "$DIR" = "." ] && gen_outfile || output="$DIR"
    #echo $output
  #else
    #output="${file}-html"
    # input_file=${file##*/}
    # output="`echo $input_file | sed 's,\.\(sgml\|sgm\|xml\)$,,'`"
  #fi
  if [ -z "$quiet" ]; then
    echo "input file was called $input_file -- output will be in $output"
  fi

  TMPDIR=$(mktemp -d db2html.XXXXXX) || exit 1
  [ -z "$quiet" ] && echo TMPDIR is $TMPDIR

  SAVE_PWD=$(pwd)
  if [ "$file" = "$(basename "$file")" ]; then
    [ -z "$quiet" ] && echo "working on ../$file"
    (cd $TMPDIR ;\
     # echo ok;\
     $JADE ${JADEHTML} ${JADEVARS} -V html-backend \
          -c $CATALOG $_CAT_JADE -D $dslhome \
          -d ${STYLE}\#html \
          -ihtml \
          -t sgml $SGML_DECL_SLOPPY ../"$file"; \
     cd $SAVE_PWD)
  else
    [ -z "$quiet" ] && echo "working on $file"
    (cd $TMPDIR; \
     $JADE ${JADEHTML} ${JADEVARS} -V html-backend \
          -c $CATALOG $_CAT_JADE -D $dslhome \
          -d ${STYLE}\#html \
          -ihtml \
          -t sgml $SGML_DECL_SLOPPY "$file"; \
     cd $SAVE_PWD)
  fi
  
  #if [ $# -eq 1 ]; then
    if [ -d "${output}.junk" ]; then
      /bin/rm -rf "${output}.junk"
    fi
    if [ -d "${output}" ]; then
      mv "$output" "${output}.junk"
    fi
    if [ -z "$quiet" ]; then
      echo "about to copy cascading stylesheet and admon graphics to temp dir"
    fi
    if [ -f ${HTML_STYLESHEET} ]; then
      cp ${HTML_STYLESHEET} ${TMPDIR}/
    else
      echo "No such file: ${HTML_STYLESHEET}"
    fi
    mkdir ${TMPDIR}/stylesheet-images
    cp ${ADMON_GRAPHICS} ${TMPDIR}/stylesheet-images
    if [ -z "$quiet" ]; then
      echo "about to rename temporary directory to \"$output\""
    fi
    mv ${TMPDIR} "$output"
  #else
  #  cat $TMPDIR/*
  #fi
  
  rm -rf $TMPDIR
} # do_html

do_mif(){
  end=mif
  gen_outfile
  echo mif output file name is "$output"
  $JADE ${JADEVARS}  -V mif-backend -t mif \
    -c $CATALOG $_CAT_JADE -D $dslhome -d ${STYLE}\#print \
    -o "$output" $SGML_DECL_SLOPPY "$file" || jade_error
} # do_mif

do_pdf(){
  texfile=$output
  echo tex file name is "$texfile"
  end=pdf
  gen_outfile
  echo pdf file name is "$output"
  pdfjadetex "$texfile" || exit 1
  
  # if there are unresolved references, re-run pdfjadetex, twice
  if egrep '^LaTeX Warning: There were undefined references.$' "${outputlog}" >/dev/null 2>&1
  then
      pdfjadetex "$texfile" || exit 1
      pdfjadetex "$texfile" || exit 1
  fi
} # do_pdf

do_ps(){
  dvifile=$output
  echo dvi file name is "$dvifile"
  if echo "$file" | egrep -i '\.sgml$|\.sgm$|\.xml$' >/dev/null 2>&1; then
    output=`echo "$file" | sed 's,\.\(sgml\|sgm\|xml\)$,.ps,'`
  fi
  echo ps file name is "$output"
  # if [ -n "`type -p dvips`" ]; then echo "*** dvips missing!" ; exit 1; fi
  dvips -o "$output" "$dvifile"
} # do_ps

do_rtf(){
  if echo "$file" | egrep -i '\.sgml$|\.sgm$|\.xml$' >/dev/null 2>&1; then
    output=`echo "$file" | sed 's,\.\(sgml\|sgm\|xml\)$,.rtf,'`
  fi
  echo rtf output file name is "$output"
  $JADE ${JADEVARS}  -V rtf-backend -t rtf \
    -c $CATALOG $_CAT_JADE -D $dslhome -d ${STYLE}\#print \
    -o "$output" $SGML_DECL_SLOPPY "$file" || jade_error
} # do_rtf

do_tex(){
  if echo "$file" | egrep -i '\.sgml$|\.sgm$|\.xml$' >/dev/null 2>&1; then
    output=`echo "$file" | sed 's,\.\(sgml\|sgm\|xml\)$,.tex,'`
  fi
  echo tex output file name is "$output"
  $JADE ${JADEVARS} -V tex-backend -t tex \
    -c $CATALOG $_CAT_JADE -D $dslhome -d ${STYLE}\#print \
    -o "$output" $SGML_DECL_SLOPPY "$file" || jade_error
} # do_tex

do_xml(){
  if echo "$file" | egrep -i '\.sgml$|\.sgm$' >/dev/null 2>&1; then
    output=`echo "$file" | sed 's,\.\(sgml\|sgm\)$,.xml,'`
  fi
  echo xml output file name is "$output"
  if [ -z "$XMLENCODING" ]; then XMLENCODING=iso-8859-1 ; fi
  extract_dtd_declaration
  xmldtddecl=$(echo -n "<!DOCTYPE $doctype " ;
      grep 'DTD DocBook XML V[[:digit:]]' /usr/share/sgml/CATALOG.db41xml ;)
  # write doctype declaration
  # grep 'DTD DocBook XML V[[:digit:]]' /usr/share/sgml/CATALOG.db41xml \
  #    | sed 's:.* \(.*\):\1:'
  {
      echo $dtd_declaration
      ${SGMLNORM} -c $CATALOG $_CAT_JADE $SGML_DECL_SLOPPY $file
  } | ${SGML2XML} -c $CATALOG $_CAT_JADE \
      -b $XMLENCODING -xempty -xlower $XMLOPTIONS \
      | sed "2i\\
$xmldtddecl" > "$output"
} #do_xml

do_validate(){
  # -m $CATALOG

  # echo nsgmls -s $CAT_SGML_NSGMLS $file
  if [ $flavor = sgml ]; then
    $NSGMLS -s $CAT_SGML_NSGMLS $SGML_DECL_SLOPPY "$file"
    [ $? = "0" ] && echo "$file is valid."
  else
    $NSGMLS -s $CAT_NSGMLS $SGML_DECL_SLOPPY "$file"
    [ $? = "0" ] && echo "$file is valid."
  fi
} #do_validate

check_pdf(){
  if [ x = x$($WHICH pdfjadetex) ]; then
    echo "'pdfjadetex' not found."
    echo "Install jadetex and required TeX packages to generate PDF."
    exit 1
  fi
} #check_pdf

check_dvi(){
  if [ x = x$($WHICH jadetex) ]; then
    echo "'jadetex' not found."
    echo "Install 'jadetex' and required TeX packages to generate DVI."
    exit 1
  fi
} #check_dvi

check_ps(){
  check_dvi
  if [ x = x$($WHICH dvips) ]; then
    echo "'dvips' not found."
    echo "Install 'dvips' to generate PostScript."
    exit 1
  fi
} #check_ps

case $mode in
  db2dvi) check_dvi; do_tex; do_dvi ;;
  db2fot) do_fot ;;
  db2html) do_html ;;
  db2mif) do_mif;;
  db2ps) check_ps; do_tex; do_dvi; do_ps ;;
  db2pdf) check_pdf; do_tex; do_pdf ;;
  db2rtf) do_rtf ;;
  db2tex) do_tex ;;
  db2xml) do_xml ;;
  validate) do_validate ;;
esac

exit 0
